/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.event
 * File:    EventService.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.event;

import java.util.ArrayList;

import de.fhhannover.inform.ifmap.communication.Connection;
import de.fhhannover.inform.ifmap.datastructure.PollResultContainer;

public class EventService {

	private ArrayList<StatusChangedReceiver> mStatusReceiver = new ArrayList<StatusChangedReceiver>();
	private ArrayList<IdentifierChangedReceiver> mIdentifierReceiver = new ArrayList<IdentifierChangedReceiver>();

	public void addStatusListener(StatusChangedReceiver r) {
		if (!mStatusReceiver.contains(r))
			mStatusReceiver.add(r);
	}

	public void addUpdateListener(IdentifierChangedReceiver r) {
		if (!mIdentifierReceiver.contains(r))
			mIdentifierReceiver.add(r);
	}

	public synchronized void notifyRepositoryChanged(Connection con, PollResultContainer prc) {
		if (prc != null) {
			for (IdentifierChangedReceiver r : mIdentifierReceiver) {
				r.processNewPollResult(con, prc.getNew(), prc.getUpdate(),
						prc.getDelete());
			}
		}
	}

	public synchronized void notifyStatusChangedReceiver(Connection con, String evt) {
		if (evt != null) {
			for (StatusChangedReceiver r : mStatusReceiver) {
				r.processStatusEvent(con, evt);
			}
		}
	}

}
